@@ -1,11 +1,13 @@
 <?php
 /**
- * $Horde: turba/data.php,v 1.70.4.11 2007/01/02 13:55:18 jan Exp $
+ * $Horde: turba/data.php,v 1.70.4.17 2008/06/13 08:40:03 jan Exp $
  *
- * Copyright 2001-2007 Jan Schneider <jan@horde.org>
+ * Copyright 2001-2008 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file LICENSE for license information (ASL).  If you
  * did not receive this file, see http://www.horde.org/licenses/asl.php.
+ *
+ * @author Jan Schneider <jan@horde.org>
  */
 
 function _cleanup()
@@ -36,6 +38,32 @@ function _emptyAttributeFilter($var)
     }
 }
 
+/**
+ * Static function to make a given email address rfc822 compliant.
+ *
+ * @param string $address  An email address.
+ *
+ * @return string  The RFC822-formatted email address.
+ */
+function _getBareEmail($address)
+{
+    // Empty values are still empty.
+    if (!$address) {
+        return $address;
+    }
+
+    require_once 'Mail/RFC822.php';
+    require_once 'Horde/MIME.php';
+
+    static $rfc822;
+    if (is_null($rfc822)) {
+        $rfc822 = new Mail_RFC822();
+    }
+
+    $rfc822->validateMailbox($address);
+    return MIME::rfc822WriteAddress($address->mailbox, $address->host);
+}
+
 @define('TURBA_BASE', dirname(__FILE__));
 require_once TURBA_BASE . '/lib/base.php';
 require_once 'Horde/Data.php';
@@ -45,12 +73,22 @@ if (!$conf['menu']['import_export']) {
     exit;
 }
 
+/* If there are absolutely no valid sources, abort. */
+if (!$cfgSources) {
+    $notification->push(_("No Address Books are currently available. Import and Export is disabled."), 'horde.error');
+    require TURBA_TEMPLATES . '/common-header.inc';
+    require TURBA_TEMPLATES . '/menu.inc';
+    require $registry->get('templates', 'horde') . '/common-footer.inc';
+    exit;
+}
+
 /* Importable file types. */
 $file_types = array('csv'      => _("CSV"),
                     'tsv'      => _("TSV"),
                     'vcard'    => _("vCard"),
                     'mulberry' => _("Mulberry Address Book"),
-                    'pine'     => _("Pine Address Book"));
+                    'pine'     => _("Pine Address Book"),
+                    'ldif'     => _("LDIF Address Book"));
 
 /* Templates for the different import steps. */
 $templates = array(
@@ -69,9 +107,6 @@ $app_fields = array();
 $time_fields = array();
 $error = false;
 $outlook_mapping = array(
-    'firstname' => 'first_name',
-    'middlename' => 'middle_name',
-    'lastname' => 'last_name',
     'e-mail' => 'email',
     'homeaddress' => 'homeAddress',
     'businessaddress' => 'workAddress',
@@ -80,21 +115,14 @@ $outlook_mapping = array(
     'mobilephone' => 'cellPhone',
     'businessfax' => 'fax',
     'jobtitle' => 'title',
-    'company' => 'company',
-    'notes' => 'notes',
-    'name' => 'name',
     'internetfreebusy' => 'freebusyUrl',
-    'nickname' => 'alias',
-    'pgpPublicKey' => 'pgpPublicKey',
-    'smimePublicKey' => 'smimePublicKey',
     );
 $param = array('time_fields' => $time_fields,
                'file_types'  => $file_types,
                'import_mapping' => $outlook_mapping);
 $import_format = Util::getFormData('import_format', '');
-$driver = $import_format;
-if ($driver == 'mulberry' || $driver == 'pine') {
-    $driver = 'tsv';
+if ($import_format == 'mulberry' || $import_format == 'pine') {
+    $import_format = 'tsv';
 }
 if ($actionID != 'select') {
     array_unshift($templates[IMPORT_FILE], TURBA_TEMPLATES . '/data/import.inc');
@@ -107,9 +135,6 @@ case 'export':
     if (Util::getFormData('selected')) {
         foreach (Util::getFormData('objectkeys') as $objectkey) {
             list($source, $key) = explode(':', $objectkey, 2);
-            if (strpos($key, ':')) {
-                list($owner, $key) = explode(':', $key, 2);
-            }
             if (!isset($sources[$source])) {
                 $sources[$source] = array();
             }
@@ -129,18 +154,18 @@ case 'export':
     $all_fields = array();
     foreach ($sources as $source => $objectkeys) {
         /* Create a Turba storage instance. */
-        $storage = &Turba_Driver::singleton($source);
-        if (is_a($storage, 'PEAR_Error')) {
-            $notification->push(sprintf(_("Failed to access the address book: %s"), $storage->getMessage()), 'horde.error');
+        $driver = &Turba_Driver::singleton($source);
+        if (is_a($driver, 'PEAR_Error')) {
+            $notification->push(sprintf(_("Failed to access the address book: %s"), $driver->getMessage()), 'horde.error');
             $error = true;
             break;
         }
 
         /* Get the full, sorted contact list. */
         if (count($objectkeys)) {
-            $results = &$storage->getObjects($objectkeys);
+            $results = &$driver->getObjects($objectkeys);
         } else {
-            $results = $storage->search(array());
+            $results = $driver->search(array());
             if (is_a($results, 'Turba_List')) {
                 $results = $results->objects;
             }
@@ -151,9 +176,9 @@ case 'export':
             break;
         }
 
-        $fields = $storage->getFields();
+        $fields = array_keys($driver->map);
         $all_fields = array_merge($all_fields, $fields);
-        $params = $storage->getParams();
+        $params = $driver->getParams();
         foreach ($results as $ob) {
             $row = array();
             foreach ($fields as $field) {
@@ -218,23 +243,28 @@ case 'export':
         $vcard = &Horde_Data::singleton('vcard');
         $vcard->exportFile(_("contacts.vcf"), $cards, true);
         exit;
+
+    case 'ldif':
+        $ldif = &Horde_Data::singleton(array('turba', 'ldif'));
+        $ldif->exportFile(_("contacts.ldif"), $data, true);
+        exit;
     }
     break;
 
 case IMPORT_FILE:
     $dest = Util::getFormData('dest');
-    $storage = &Turba_Driver::singleton($dest);
-    if (is_a($storage, 'PEAR_Error')) {
-        $notification->push(sprintf(_("Failed to access the address book: %s"), $storage->getMessage()), 'horde.error');
+    $driver = &Turba_Driver::singleton($dest);
+    if (is_a($driver, 'PEAR_Error')) {
+        $notification->push(sprintf(_("Failed to access the address book: %s"), $driver->getMessage()), 'horde.error');
         $error = true;
         break;
     }
 
     /* Check permissions. */
-    $max_contacts = Turba::hasPermission($dest . ':max_contacts', 'source');
+    $max_contacts = Turba::getExtendedPermission($driver, 'max_contacts');
     if ($max_contacts !== true &&
-        $max_contacts <= $storage->countContacts()) {
-        $message = @htmlspecialchars(sprintf(_("You are not allowed to create more than %d contacts in \"%s\"."), $max_contacts, $storage->title), ENT_COMPAT, NLS::getCharset());
+        $max_contacts <= $driver->count()) {
+        $message = @htmlspecialchars(sprintf(_("You are not allowed to create more than %d contacts in \"%s\"."), $max_contacts, $driver->title), ENT_COMPAT, NLS::getCharset());
         if (!empty($conf['hooks']['permsdenied'])) {
             $message = Horde::callHook('_perms_hook_denied', array('turba:max_contacts'), 'horde', $message);
         }
@@ -263,8 +293,12 @@ case IMPORT_DATETIME:
     break;
 }
 
-if (!$error && !empty($driver)) {
-    $data = &Horde_Data::singleton($driver);
+if (!$error && !empty($import_format)) {
+    if ($import_format == 'ldif') {
+        $data = &Horde_Data::singleton(array('turba', $import_format));
+    } else {
+        $data = &Horde_Data::singleton($import_format);
+    }
     if (is_a($data, 'PEAR_Error')) {
         $notification->push(_("This file format is not supported."), 'horde.error');
         $next_step = IMPORT_FILE;
@@ -297,22 +331,16 @@ if (is_array($next_step)) {
 
     /* Create a Turba storage instance. */
     $dest = $_SESSION['import_data']['target'];
-    $storage = &Turba_Driver::singleton($dest);
-    if (is_a($storage, 'PEAR_Error')) {
-        $notification->push(sprintf(_("Failed to access the address book: %s"), $storage->getMessage()), 'horde.error');
+    $driver = &Turba_Driver::singleton($dest);
+    if (is_a($driver, 'PEAR_Error')) {
+        $notification->push(sprintf(_("Failed to access the address book: %s"), $driver->getMessage()), 'horde.error');
     } elseif (!count($next_step)) {
         $notification->push(sprintf(_("The %s file didn't contain any contacts."),
                                     $file_types[$_SESSION['import_data']['format']]), 'horde.error');
     } else {
         /* Purge old address book if requested. */
         if ($_SESSION['import_data']['purge']) {
-            if (strpos($dest, ':')) {
-                list($sourceType, $uid) = explode(':', $dest, 2);
-                $owner = $uid;
-            } else {
-                $owner = Auth::getAuth();
-            }
-            $result = $storage->deleteAll($owner);
+            $result = $driver->deleteAll();
             if (is_a($result, 'PEAR_Error')) {
                 $notification->push(sprintf(_("The address book could not be purged: %s"), $result->getMessage()), 'horde.error');
             } else {
@@ -323,12 +351,12 @@ if (is_array($next_step)) {
         $error = false;
         foreach ($next_step as $row) {
             if (is_a($row, 'Horde_iCalendar_vcard')) {
-                $row = $storage->toHash($row);
+                $row = $driver->toHash($row);
             }
 
             /* Don't search for empty attributes. */
             $row = array_filter($row, '_emptyAttributeFilter');
-            $result = $storage->search($row);
+            $result = $driver->search($row);
             if (is_a($result, 'PEAR_Error')) {
                 $notification->push($result, 'horde.error');
                 $error = true;
@@ -338,13 +366,14 @@ if (is_array($next_step)) {
                 $object = $result->next();
                 $notification->push(sprintf(_("\"%s\" already exists and was not imported."), $object->getValue('name')), 'horde.message');
             } else {
-                /* Get share information. */
-                if ($storage->usingShares) {
-                    $row['__owner'] = $storage->share->get('uid');
-                } else {
-                    $row['__owner'] = Auth::getAuth();
+                /* Check for, and validate, any email fields */
+                foreach (array_keys($row) as $field) {
+                    if ($attributes[$field]['type'] == 'email') {
+                        $row[$field] = _getBareEmail($row[$field]);
+                    }
                 }
-                $result = $storage->add($row);
+                $row['__owner'] = $driver->getContactOwner();
+                $result = $driver->add($row);
                 if (is_a($result, 'PEAR_Error')) {
                     $notification->push(sprintf(_("There was an error importing the data: %s"),
                                                 $result->getMessage()), 'horde.error');
