@@ -5,12 +5,13 @@ require_once 'Horde/Cache.php';
 /**
  * Turba directory driver implementation for an IMSP server.
  *
- * $Horde: turba/lib/Driver/imsp.php,v 1.21.4.15 2007/08/05 19:30:08 mrubinsk Exp $
+ * $Horde: turba/lib/Driver/imsp.php,v 1.21.4.21 2008/04/09 20:30:53 mrubinsk Exp $
  *
  * @author  Michael Rubinsky <mrubinsk@horde.org>
  * @package Turba
  */
 class Turba_Driver_imsp extends Turba_Driver {
+
     /**
      * Handle for the IMSP connection.
      *
@@ -53,6 +54,10 @@ class Turba_Driver_imsp extends Turba_Driver {
      */
     var $_noGroups = '';
 
+    var $_capabilities = array(
+        'delete_all' => true,
+        'delete_addressbook' => true
+    );
 
     /**
      * Constructs a new Turba imsp driver object.
@@ -61,11 +66,9 @@ class Turba_Driver_imsp extends Turba_Driver {
      */
     function Turba_Driver_imsp($params)
     {
-        $this->type         = 'imsp';
         $this->params       = $params;
         $this->_groupField  = $params['group_id_field'];
         $this->_groupValue  = $params['group_id_value'];
-        $this->_bookName    = $params['name'];
         $this->_myRights    = $params['my_rights'];
         $this->_perms       = $this->_aclToHordePerms($params['my_rights']);
     }
@@ -75,8 +78,10 @@ class Turba_Driver_imsp extends Turba_Driver {
      */
     function _init()
     {
-        global $prefs, $conf;
+        global $conf;
         require_once 'Net/IMSP.php';
+
+        $this->_bookName = $this->getContactOwner();
         $this->_imsp = &Net_IMSP::singleton('Book', $this->params);
         $result = $this->_imsp->init();
         if (is_a($result, 'PEAR_Error')) {
@@ -92,6 +97,7 @@ class Turba_Driver_imsp extends Turba_Driver {
             }
         }
 
+        Horde::logMessage('IMSP Driver initialized for ' . $this->_bookName, __FILE__, __LINE__, PEAR_LOG_DEBUG);
         $this->_authenticated = true;
         return true;
     }
@@ -114,7 +120,6 @@ class Turba_Driver_imsp extends Turba_Driver {
         }
 
         /* Get the search criteria. */
-        $imspSearch = array();
         if (count($criteria)) {
             foreach ($criteria as $key => $vals) {
                 if (strval($key) == 'OR') {
@@ -126,22 +131,9 @@ class Turba_Driver_imsp extends Turba_Driver {
         }
 
         /* Now we have a list of names, get the rest. */
-        $namesCount = count($names);
-        for ($i = 0; $i < $namesCount; $i++) {
-            $temp = $this->_read('name', array($names[$i]), $fields);
-            if (is_a($temp, 'PEAR_Error')) {
-                $GLOBALS['notification']->push($temp, 'horde.error');
-            } else {
-                $result = $temp[0];
-                if (is_a($result, 'PEAR_Error')) {
-                    $GLOBALS['notification']->push($results, 'horde.error');
-                } elseif (($this->_noGroups) && (isset($result[$this->_groupField])) &&
-                          ($result[$this->_groupField]) == $this->_groupValue) {
-                    unset($result);
-                } else {
-                    $results[] = $result;
-                }
-            }
+        $result = $this->_read('name', $names, null, $fields);
+        if (is_array($result)) {
+            $results = $result;
         }
 
         Horde::logMessage(sprintf('IMSP returned %s results',
@@ -151,30 +143,36 @@ class Turba_Driver_imsp extends Turba_Driver {
 
     /**
      * Reads the given data from the IMSP server and returns the
-     * result's fields.
+     * results.
      *
-     * @param array $criteria  (Ignored: Always 'name' for IMSP) Search criteria.
-     * @param array $id        Array of data identifiers.
-     * @param array $fields    List of fields to return.
+     * @param string $key    The primary key field to use (always 'name' for IMSP).
+     * @param mixed $ids     The ids of the contacts to load.
+     * @param string $owner  Only return contacts owned by this user.
+     * @param array $fields  List of fields to return.
      *
      * @return array  Hash containing the search results.
      */
-    function _read($criteria, $id, $fields)
+    function _read($key, $ids, $owner, $fields)
     {
         $results = array();
         if (!$this->_authenticated) {
             return $results;
         }
-        $id = array_values($id);
-        $idCount = count($id);
-        $result = array();
+        $ids = array_values($ids);
+        $idCount = count($ids);
         $members = array();
         $tmembers = array();
+        $IMSPGroups = array();
 
         for ($i = 0; $i < $idCount; $i++) {
-            $temp = $this->_imsp->getEntry($this->_bookName, $id[$i]);
+            $result = array();
+            if (!isset($IMSPGroups[$ids[$i]])) {
+                $temp = $this->_imsp->getEntry($this->_bookName, $ids[$i]);
+            } else {
+                $temp = $IMSPGroups[$ids[$i]];
+            }
             if (is_a($temp, 'PEAR_Error')) {
-                return $temp;
+                continue;
             } else {
                 $temp['fullname'] = $temp['name'];
                 $isIMSPGroup = false;
@@ -184,26 +182,64 @@ class Turba_Driver_imsp extends Turba_Driver {
 
                 if ((isset($temp[$this->_groupField])) &&
                     ($temp[$this->_groupField] == $this->_groupValue)) {
+                    if ($this->_noGroups) {
+                        continue;
+                    }
+                    if (!isset($IMSPGroups[$ids[$i]])) {
+                        $IMSPGroups[$ids[$i]] = $temp;
+                    }
+                    // move group ids to end of list
+                    if ($idCount > count($IMSPGroups) &&
+                        $idCount - count($IMSPGroups) > $i) {
+                        $ids[] = $ids[$i];
+                        unset($ids[$i]);
+                        $ids = array_values($ids);
+                        $i--;
+                        continue;
+                    }
                     $isIMSPGroup = true;
                 }
                 // Get the group members that might have been added from other
-                // IMSP applications.
-                if ($isIMSPGroup) {
+                // IMSP applications, but only if we need more information than
+                // the group name
+                if ($isIMSPGroup &&
+                    array_search('__members', $fields) !== false) {
+
                     if (isset($temp['email'])) {
                         $emailList = $this->_getGroupEmails($temp['email']);
                         $count = count($emailList);
                         for ($j = 0; $j < $count; $j++) {
-                            $memberName = $this->_imsp->search
-                                ($this->_bookName, array('email' => trim($emailList[$j])));
+                            $needMember = true;
+                            foreach ($results as $curResult) {
+                                if (!empty($curResult['email']) &&
+                                    strtolower($emailList[$j]) ==
+                                      strtolower(trim($curResult['email']))) {
+                                    $members[] = $curResult['name'];
+                                    $needMember = false;
+                                }
+                            }
+                            if ($needMember) {
+                                $memberName = $this->_imsp->search
+                                    ($this->_bookName,
+                                     array('email' => trim($emailList[$j])));
 
-                            if (count($memberName)) {
-                                $members[] = $memberName[0];
+                                if (count($memberName)) {
+                                    $members[] = $memberName[0];
+                                }
                             }
                         }
                     }
-                    if (isset($temp['__members'])) {
+                    if (!empty($temp['__members'])) {
                         $tmembers = @unserialize($temp['__members']);
                     }
+
+                    // TODO: Make sure that we are using the correct naming
+                    // convention for members regardless of if we are using
+                    // shares or not. This is needed to assure groups created
+                    // while not using shares won't be lost when transitioning
+                    // to shares and visa versa.
+                    //$tmembers = $this->_checkMemberFormat($tmembers);
+
                     $temp['__members'] = serialize($this->_removeDuplicated(
                                                    array($members, $tmembers)));
                     $temp['__type'] = 'Group';
@@ -223,6 +259,9 @@ class Turba_Driver_imsp extends Turba_Driver {
             $results[] = $result;
         }
 
+        if (empty($results) && isset($temp) && is_a($temp, 'PEAR_Error')) {
+          return $temp;
+        }
         return $results;
     }
 
@@ -241,7 +280,6 @@ class Turba_Driver_imsp extends Turba_Driver {
                 $attributes['__members'] = '';
                 $attributes['email'] = ' ';
             }
-
             $temp = unserialize($attributes['__members']);
             if (is_array($temp)) {
                 $members = array_values($temp);
@@ -249,16 +287,20 @@ class Turba_Driver_imsp extends Turba_Driver {
                 $members = array();
             }
 
-            if (count($members)) {
-                $result = $this->_read('name', $members, array('email'));
-                if (!is_a($result, 'PEAR_Error')) {
-                    $count = count($result);
-                    for ($i = 0; $i < $count; $i++) {
-                        if (isset($result[$i]['email'])) {
-                            $contact = sprintf("%s<%s>\n", $members[$i],
-                                               $result[$i]['email']);
-                            $attributes['email'] .= $contact;
-                        }
+            // This searches the current IMSP address book to see if
+            // we have a match for this member before adding to email
+            // attribute since IMSP groups in other IMSP aware apps
+            // generally require an existing conact entry in the current
+            // address book for each group member (this is necessary for
+            // those sources that may be used both in AND out of Horde).
+            $result = $this->_read('name', $members, null, array('email'));
+            if (!is_a($result, 'PEAR_Error')) {
+                $count = count($result);
+                for ($i = 0; $i < $count; $i++) {
+                    if (isset($result[$i]['email'])) {
+                        $contact = sprintf("%s<%s>\n", $members[$i],
+                                           $result[$i]['email']);
+                        $attributes['email'] .= $contact;
                     }
                 }
             }
@@ -281,6 +323,17 @@ class Turba_Driver_imsp extends Turba_Driver {
         return $this->_imsp->deleteEntry($this->_bookName, $object_id);
     }
 
+    /**
+     * Deletes the address book represented by this driver from the IMSP server.
+     *
+     * @return mixed  true | PEAR_Error
+     * @since Turba 2.2
+     */
+     function _deleteAll()
+     {
+         $this->_imsp->deleteAddressbook($this->_bookName);
+     }
+
     /**
      * Saves the specified object to the IMSP server.
      *
@@ -300,7 +353,6 @@ class Turba_Driver_imsp extends Turba_Driver {
             $attributes['name'] = $this->_makeKey($attributes);
             $object_id = $attributes['name'];
         }
-
         $result = $this->_add($attributes);
         return is_a($result, 'PEAR_Error') ? $result : $object_id;
     }
@@ -389,6 +441,7 @@ class Turba_Driver_imsp extends Turba_Driver {
     function _sendSearch($criteria)
     {
         global $conf;
+
         $names = '';
         $imspSearch = array();
         $searchkey = $criteria['field'];
@@ -465,7 +518,7 @@ class Turba_Driver_imsp extends Turba_Driver {
         $results = $cache->get($key, 15);
 
         if ($results) {
-            $names = explode(',', $results);
+            $names = unserialize($results);
         }
 
         if (!$names) {
@@ -473,7 +526,7 @@ class Turba_Driver_imsp extends Turba_Driver {
             if (is_a($names, 'PEAR_Error')) {
                 $GLOBALS['notification']->push($names, 'horde.error');
             } else {
-                $cache->set($key, implode(",", $names));
+                $cache->set($key, serialize($names));
                 return $names;
             }
         } else {
@@ -497,7 +550,9 @@ class Turba_Driver_imsp extends Turba_Driver {
             return $names[0];
         } else {
             for ($i = 0; $i < count($names); $i++) {
-                $results = array_merge($results, $names[$i]);
+                if (is_array($names[$i])) {
+                    $results = array_merge($results, $names[$i]);
+                }
             }
             $search = array_count_values($results);
             foreach ($search as $key => $value) {
@@ -536,33 +591,102 @@ class Turba_Driver_imsp extends Turba_Driver {
      *
      * @return boolean  true if user has permission, false otherwise.
      */
-     function hasPermission($perm)
-     {
-         return $this->_perms & $perm;
-     }
+    function hasPermission($perm)
+    {
+        return $this->_perms & $perm;
+    }
+
+    /**
+     * Converts an acl string to a Horde Permissions bitmask.
+     *
+     * @param string $acl  A standard, IMAP style acl string.
+     *
+     * @return integer  Horde Permissions bitmask.
+     */
+    function _aclToHordePerms($acl)
+    {
+        $hPerms = 0;
+        if (strpos($acl, 'w') !== false) {
+            $hPerms |= PERMS_EDIT;
+        }
+        if (strpos($acl, 'r') !== false) {
+            $hPerms |= PERMS_READ;
+        }
+        if (strpos($acl, 'd') !== false) {
+            $hPerms |= PERMS_DELETE;
+        }
+        if (strpos($acl, 'l') !== false) {
+            $hPerms |= PERMS_SHOW;
+        }
+        return $hPerms;
+    }
+
+    /**
+     * Creates a new Horde_Share and creates the address book
+     * on the IMSP server.
+     *
+     * @param array  The params for the share.
+     *
+     * @return mixed  The share object or PEAR_Error.
+     * @since Turba 2.2
+     */
+    function &createShare($share_id, $params)
+    {
+        if (isset($params['default']) && $params['default'] === true) {
+            $params['params']['name'] = $this->params['username'];
+        } else {
+            $params['params']['name'] = $this->params['username'] . '.' . $params['name'];
+        }
+        $result = &Turba::createShare($share_id, $params);
+        if (is_a($result, 'PEAR_Error')) {
+            return $result;
+        }
+
+        $imsp_result = Net_IMSP_Utils::createBook($GLOBALS['cfgSources']['imsp'], $params['params']['name']);
+        if (is_a($imsp_result, 'PEAR_Error')) {
+            return $imsp_result;
+        }
+        return $result;
+    }
+
+    /**
+     * Helper function to count the occurances of the ':'
+     * delimter in group member entries.
+     *
+     * @param string $in  The group member entry.
+     *
+     * @return integer  The number of ':' in $in.
+     */
+    function _countDelimiters($in)
+    {
+        $cnt = 0;
+        $pos = 0;
+        $i = -1;
+        while (($pos = strpos($in, ':', $pos + 1)) !== false) {
+            ++$cnt;
+        }
+        return $cnt;
+    }
+
+    /**
+     * Returns the owner for this contact. For an IMSP source, this should be
+     * the name of the address book.
+     */
+    function _getContactOwner()
+    {
+       return $this->params['name'];
+    }
+
+    function checkDefaultShare(&$share, $srcConfig)
+    {
+        $params = @unserialize($share->get('params'));
+        if (!isset($params['default'])) {
+            $params['default'] = ($params['name'] == $srcConfig['params']['username']);
+            $share->set('params', serialize($params));
+            $share->save();
+        }
+
+        return $params['default'];
+    }
 
-     /**
-      * Converts an acl string to a Horde Permissions bitmask.
-      *
-      * @param string $acl  A standard, IMAP style acl string.
-      *
-      * @return integer  Horde Permissions bitmask.
-      */
-     function _aclToHordePerms($acl)
-     {
-         $hPerms = 0;
-         if (strpos($acl, 'w') !== false) {
-             $hPerms |= PERMS_EDIT;
-         }
-         if (strpos($acl, 'r') !== false) {
-             $hPerms |= PERMS_READ;
-         }
-         if (strpos($acl, 'd') !== false) {
-             $hPerms |= PERMS_DELETE;
-         }
-         if (strpos($acl, 'l') !== false) {
-             $hPerms |= PERMS_SHOW;
-         }
-         return $hPerms;
-     }
 }
