@@ -1,11 +1,13 @@
 <?php
 /**
- * $Horde: turba/test.php,v 1.9.10.4 2006/01/01 21:29:15 jan Exp $
+ * $Horde: turba/test.php,v 1.9.10.8 2008/01/02 11:32:38 jan Exp $
  *
- * Copyright 2000-2006 Brent J. Nordquist <bjn@horde.org>
+ * Copyright 1999-2008 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file LICENSE for license information (ASL).  If you
  * did not receive this file, see http://www.horde.org/licenses/asl.php.
+ *
+ * @author Brent J. Nordquist <bjn@horde.org>
  */
 
 /* Include Horde's core.php file. */
@@ -15,17 +17,17 @@ include_once '../lib/core.php';
  * package, in core.php. If String:: isn't defined, then we're not
  * finding some critical libraries. */
 if (!class_exists('String')) {
-    echo '<br /><span style="color: red; font-size: 18px; font-weight: bold;">The Horde_Util package was not found. If PHP\'s error_reporting setting is high enough, there should be error messages printed above that may help you in debugging the problem. If you are simply missing these files, then you need to get the <a href="http://cvs.horde.org/cvs.php/framework">framework</a> module from <a href="http://horde.org/source/">Horde CVS</a>, and install the packages in it with the install-packages.php script.</span>';
+    echo '<br /><h2 style="color:red">The Horde_Util package was not found. If PHP\'s error_reporting setting is high enough and display_errors is on, there should be error messages printed above that may help you in debugging the problem. If you are simply missing these files, then you need to get the <a href="http://cvs.horde.org/cvs.php/framework">framework</a> module from <a href="http://www.horde.org/source/">Horde CVS</a>, and install the packages in it with the install-packages.php script.</h2>';
     exit;
 }
 
 /* Initialize the Horde_Test:: class. */
-if (!(@is_readable('../lib/Test.php'))) {
+if (!is_readable('../lib/Test.php')) {
     echo 'ERROR: You must install Horde before running this script.';
     exit;
 }
 require_once '../lib/Test.php';
-$horde_test = &new Horde_Test;
+$horde_test = new Horde_Test;
 
 /* Turba version. */
 $module = 'Turba';
@@ -49,14 +51,28 @@ $module_list = array(
     'ldap' => 'LDAP Support'
 );
 
+/* PEAR packages. */
+$pear_list = array(
+    'Net_LDAP' => array(
+        'path' => 'Net/LDAP.php',
+        'error' => 'Net_LDAP is required when doing schema checks with LDAP address books.',
+    ),
+);
+
 /* Get the status output now. */
 $module_output = $horde_test->phpModuleCheck($module_list);
+$pear_output = $horde_test->PEARModuleCheck($pear_list);
 
 ?>
 
 <h1>PHP Module Capabilities</h1>
 <ul>
-    <?php echo $module_output ?>
+ <?php echo $module_output ?>
+</ul>
+
+<h1>PEAR</h1>
+<ul>
+ <?php echo $pear_output ?>
 </ul>
 
 <h1>PHP LDAP Support Test</h1>
@@ -115,19 +131,16 @@ if (!empty($server) && !empty($basedn) && !empty($filter)) {
     ?>
 <form name="form1" method="post" action="test.php">
 <table>
-<tr><td align="right">Server</td><td><input type="text" name="server" /></td></tr>
-<tr><td align="right">Port</td><td><input type="text" name="port" /></td><td>(defaults to "389")</td></tr>
-<tr><td align="right">Base DN</td><td><input type="text" name="basedn" /></td><td>(e.g. "dc=example,dc=com")</td></tr>
-<tr><td align="right">User</td><td><input type="text" name="user" /></td><td>(leave blank for anonymous)</td></tr>
-<tr><td align="right">Password</td><td><input type="password" name="passwd" /></td></tr>
-<tr><td align="right">Filter</td><td><input type="text" name="filter" /></td><td>(e.g. "cn=Babs Jensen")</td></tr>
+<tr><td align="right"><label for="server">Server</label></td><td><input type="text" id="server" name="server" /></td></tr>
+<tr><td align="right"><label for="port">Port</label></td><td><input type="text" id="port" name="port" /></td><td>(defaults to "389")</td></tr>
+<tr><td align="right"><label for="basedn">Base DN</label></td><td><input type="text" id="basedn" name="basedn" /></td><td>(e.g. "dc=example,dc=com")</td></tr>
+<tr><td align="right"><label for="user">User</label></td><td><input type="text" id="user" name="user" /></td><td>(leave blank for anonymous)</td></tr>
+<tr><td align="right"><label for="passwd">Password</label></td><td><input type="password" id="passwd" name="passwd" /></td></tr>
+<tr><td align="right"><label for="filter">Filter</label></td><td><input type="text" id="filter" name="filter" /></td><td>(e.g. "cn=Babs Jensen")</td></tr>
 <tr><td></td><td><input type="submit" name="f_submit" value="Submit" /><input type="reset" name="f_reset" value="Reset" /></td></tr>
 </table>
 </form>
 <?php } ?>
 
-</td></tr>
-</table>
-
 <?php
 require TEST_TEMPLATES . 'footer.inc';
